# -*- encoding: utf-8 -*-
require 'spec_helper'

describe ResourcePicture do
  before do
    Factory :admin
    @user = Factory( :user_yas)
    @author = @user.author
    @artist = Factory :artist_yas, :author_id => @author.id
    @other_user = Factory( :user_yas)
    @other_author = @other_user.author
    @other_artist = Factory :artist_yas, :author_id => @other_author.id
    @license = Factory :license
    @original_picture = Factory :original_picture, :artist_id => @artist.id, :license_id => @license.id
    
    class Mgk ; class Image ; end ; end
    Mgk::Image.stub(:from_blob).with(any_args).and_return([Mgk.new])
    Mgk.any_instance.stub(:format).with(any_args).and_return('png')
    Mgk.any_instance.stub(:format=).with(any_args)
    Mgk.any_instance.stub(:rows).with(any_args).and_return(200)
    Mgk.any_instance.stub(:columns).with(any_args).and_return(100)
    Mgk.any_instance.stub(:filesize).with(any_args).and_return(20000)
    Mgk.any_instance.stub(:to_blob).with(any_args).and_return('data')
    #原画ファイル削除だけは必ず成功するものとしておく
    PictureIO::LocalPicture.any_instance.stub(:delete).with(any_args).and_return(true)
  end
  
  describe '検証に於いて' do
    before do
    end
    
    it 'オーソドックスなデータなら通る' do
      @rp = Factory.build :resource_picture, :artist_id => @artist.id, :original_picture_id => @original_picture.id, :license_id => @license.id
      @rp.should be_valid
    end
    
    context 'extを検証するとき' do
      before do
        @rp = Factory.build :resource_picture, 
          :artist_id => @artist.id, :original_picture_id => @original_picture.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @rp.ext = 'jpeg'
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.ext = ''
        @rp.should_not be_valid
      end
      it '5文字以上なら失敗する' do
        @rp.ext = 'a'*5
        @rp.should_not be_valid
      end
      it 'png,gif,jpeg以外なら失敗する' do
        @rp.ext = 'bmp'
        @rp.should_not be_valid
      end
    end
    context 'widthを検証するとき' do
      before do
        @rp = Factory.build :resource_picture, 
          :artist_id => @artist.id, :original_picture_id => @original_picture.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @rp.width = 1
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.width = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.width = 'a'
        @rp.should_not be_valid
      end
      it '0なら失敗する' do
        @rp.width = '0'
        @rp.should_not be_valid
      end
      it '負でも失敗する' do
        @rp.width = -1
        @rp.should_not be_valid
      end
    end
    context 'heightを検証するとき' do
      before do
        @rp = Factory.build :resource_picture, 
          :artist_id => @artist.id, :original_picture_id => @original_picture.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @rp.height = '1'
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.height = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.height = 'a'
        @rp.should_not be_valid
      end
      it '0なら失敗する' do
        @rp.height = '0'
        @rp.should_not be_valid
      end
      it '負でも失敗する' do
        @rp.height = -1
        @rp.should_not be_valid
      end
    end
    context 'filesizeを検証するとき' do
      before do
        @rp = Factory.build :resource_picture, 
          :artist_id => @artist.id, :original_picture_id => @original_picture.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @rp.filesize = '1'
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.filesize = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.filesize = 'a'
        @rp.should_not be_valid
      end
      it '負なら失敗する' do
        @rp.filesize = '-1'
        @rp.should_not be_valid
      end
      it '2MB以上なら失敗する' do
        @rp.filesize = 2000000+1
        @rp.should_not be_valid
      end
    end
    context 'artist_idを検証するとき' do
      before do
        @rp = Factory.build :resource_picture, 
          :original_picture_id => @original_picture.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @rp.artist_id = @artist.id
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.artist_id = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.artist_id = 'a'
        @rp.should_not be_valid
      end
      it '存在する絵師でなければ失敗する' do
        @rp.artist_id = 0
        @rp.should_not be_valid
      end
    end
    context 'license_idを検証するとき' do
      before do
        @rp = Factory.build :resource_picture, 
          :artist_id => @artist.id, :original_picture_id => @original_picture.id
      end
      it 'テストデータの確認' do
        @rp.license_id = @license.id
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.license_id = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.license_id = 'a'
        @rp.should_not be_valid
      end
      it '存在する絵師でなければ失敗する' do
        @rp.license_id = 0
        @rp.should_not be_valid
      end
    end
    context 'original_picture_idを検証するとき' do
      before do
        @rp = Factory.build :resource_picture, 
          :artist_id => @artist.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @rp.original_picture_id = @original_picture.id
        @rp.should be_valid
      end
      it 'nullなら失敗する' do
        @rp.original_picture_id = nil
        @rp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @rp.original_picture_id = 'a'
        @rp.should_not be_valid
      end
      it '存在する原画でなければ失敗する' do
        @rp.original_picture_id = 0
        @rp.should_not be_valid
      end
    end
  end
  
  describe 'データ補充に於いて' do
    before do
    end
    
  end
  
  describe '単体取得に於いて' do
    before do
      @op = Factory :original_picture, :artist_id => @artist.id, :license_id => @license.id
      @rp = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id
    end
    it '指定の素材を返す' do
      pic = ResourcePicture.show @rp.id
      pic.should eq @rp
    end
    context '関連テーブルオプションがないとき' do
      it 'ライセンスと絵師を含んでいる' do
        r = ResourcePicture.show_include_opt
        r.should eq [:license, :artist]
      end
    end
    context '関連テーブルオプションで原画を含ませたとき' do
      it 'ライセンスと絵師と原画データを含んでいる' do
        r = ResourcePicture.show_include_opt(:include => :original_picture)
        r.should eq [:license, :artist, :original_picture]
      end
    end
    context '存在しない素材を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          pic = ResourcePicture.show 0
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe 'json単体出力オプションに於いて' do
    it 'includeキーがライセンスと絵師を含んでいる' do
      r = ResourcePicture.show_json_include_opt
      r[:include].should eq [:license, :artist]
    end
  end
  describe '一覧取得に於いて' do
    before do
      @op = Factory :original_picture, :artist_id => @artist.id, :license_id => @license.id
      @rp = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        ResourcePicture.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        ResourcePicture.page().should eq 1
      end
      it '0以下の場合は1になる' do
        ResourcePicture.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        ResourcePicture.page_size('7').should eq 7
      end
      it 'nilの場合はResourcePicture.default_page_sizeになる' do
        ResourcePicture.page_size().should eq ResourcePicture.default_page_size
      end
      it '0以下の場合はResourcePicture.default_page_sizeになる' do
        ResourcePicture.page_size('0').should eq ResourcePicture.default_page_size
      end
      it 'ResourcePicture.max_page_sizeを超えた場合はResourcePicture.max_page_sizeになる' do
        ResourcePicture.page_size('1000').should eq ResourcePicture.max_page_size
      end
    end
    it 'リストを返す' do
      pic = ResourcePicture.list
      pic.should eq [@rp]
    end
    it '時系列で並んでいる' do
      nop = Factory :original_picture, :artist_id => @artist.id, :license_id => @license.id
      nrp = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => nop.id
      pic = ResourcePicture.list
      pic.should eq [nrp, @rp]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        nop2 = Factory :original_picture, :artist_id => @artist.id, :license_id => @license.id
        @nrp2 = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => nop2.id
        nop3 = Factory :original_picture, :artist_id => @artist.id, :license_id => @license.id
        @nrp3 = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => nop3.id
        nop4 = Factory :original_picture, :artist_id => @artist.id, :license_id => @license.id
        @nrp4 = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => nop4.id
        nop5 = Factory :original_picture, :artist_id => @artist.id, :license_id => @license.id
        @nrp5 = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => nop5.id
        ResourcePicture.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        pic = ResourcePicture.list
        pic.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        pic = ResourcePicture.list({}, 1)
        pic.should eq [@nrp5, @nrp4]
      end
      it 'page=2なら中間2件を返す' do
        pic = ResourcePicture.list({}, 2)
        pic.should eq [@nrp3, @nrp2]
      end
      it 'page=3なら先頭1件を返す' do
        pic = ResourcePicture.list({}, 3)
        pic.should eq [@rp]
      end
    end
  end
  describe 'json一覧出力オプションに於いて' do
    it 'includeキーがライセンスと絵師を含んでいる' do
      r = ResourcePicture.list_json_opt
      r[:include].should eq [:license, :artist]
    end
  end
  
  describe 'サイズの調整に於いて' do
    before do
    end
    
    it 'サイズに収まるときはそのまま使う' do
      ResourcePicture.fix_size_both(64, 64, 64, 64).should eq [64, 64]
    end
    it '小さくても拡大しない' do
      ResourcePicture.fix_size_both(64, 64, 32, 32).should eq [32, 32]
    end
    it '縦長のときは縦に合わせて縮小' do
      ResourcePicture.fix_size_both(64, 64, 64, 128).should eq [32, 64]
    end
    it '横長のときは横に合わせて縮小' do
      ResourcePicture.fix_size_both(64, 64, 128, 64).should eq [64, 32]
    end
  end
  
  describe 'フォーマット変換対象判定に於いて' do
    before do
      @rp = Factory.build :resource_picture, 
        :artist_id => @artist.id, :license_id => @license.id
    end
    context '変換するケース' do
      it '画像フォーマットがpngかつライセンスの変換禁止フラグが無効のときTrue' do
        License.any_instance.stub(:no_convert).with(any_args).and_return(0)
        @rp.ext = 'png'
        @rp.to_gif?.should be_true
      end
    end
    context '変換しないケース' do
      it '画像フォーマットがでない' do
        License.any_instance.stub(:no_convert).with(any_args).and_return(0)
        @rp.ext = 'gif'
        @rp.to_gif?.should be_false
      end
      it 'ライセンスの変換禁止フラグが無効' do
        License.any_instance.stub(:no_convert).with(any_args).and_return(1)
        @rp.ext = 'png'
        @rp.to_gif?.should be_false
      end
    end
  end
  
  describe 'Gifフォーマット変換に於いて' do
    before do
      Magick::Image.stub(:from_blob).with(any_args).and_return([Mgk.new])
    end
    context 'つつがなく終わるとき' do
      it 'Mgkオブジェクトが返る' do
        mgk = ResourcePicture.png_to_gif('mgkbin')
        mgk.is_a?(Mgk).should be_true
      end
      it 'Mgkオブジェクトはgif変換されている' do
        #スタブばかりで変換できないので代入されているかでチェックする
        Mgk.any_instance.should_receive(:format=).with('gif').exactly(1)
        ResourcePicture.png_to_gif('mgkbin')
      end
    end
    context 'RMagick変換が失敗したとき' do
      before do
        Magick::Image.should_receive(:from_blob).with(any_args).and_raise('StandardError')
      end
      it 'falseを返す' do
        res = ResourcePicture.png_to_gif('mgkbin')
        res.should be_false
      end
    end
  end
  
  describe '対象素材の取得に於いて' do
    before do
      @op = Factory.build :original_picture, :ext => 'jpeg', :width => 264, :height => 265, :filesize => 266, 
        :artist_id => @artist.id, :license_id => @license.id
      @rp = Factory.build :resource_picture, 
        :artist_id => @artist.id, :license_id => @license.id
    end
    context '原画オブジェクトが素材を持っている(更新ケース)' do
      before do
        OriginalPicture.any_instance.stub(:resource_picture).with(any_args).and_return(@rp)
      end
      it 'それを対象素材として返す' do
        res = ResourcePicture.update_picture(@op)
        res.should eq @rp
      end
    end
    context '持っていない(新規作成ケース)' do
      before do
        OriginalPicture.any_instance.stub(:resource_picture).with(any_args).and_return(nil)
      end
      it '新規素材モデルを用意し、それを対象素材として返す' do
        res = ResourcePicture.update_picture(@op)
        res.should be_a_new ResourcePicture
      end
      it '原画オブジェクトから属性を取り出して対象素材にセットしている' do
        res = ResourcePicture.update_picture(@op)
        res.ext.should eq 'jpeg'
        res.width.should eq 264
        res.height.should eq 265
        res.filesize.should eq 266
        res.artist_id.should eq @artist.id
        res.license_id.should eq @license.id
      end
      
    end
  end
  
  describe '作成・更新に於いて' do
    before do
      @op = Factory :original_picture, :artist_id => @artist.id, :license_id => @license.id
      @rp = Factory.build :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id
    end
    context '事前チェック' do
      before do
        #すべての処理が正常パターンで通過すれば、一番深い分岐まで通る。
        #それで外部のメソッド呼び出しだけに着目してテストする。
        ResourcePicture.any_instance.stub(:save).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        ResourcePicture.stub(:to_gif?).with(any_args).and_return(true)
        class GifMgk < Mgk ; end  #store_pictureは二回呼び出される。区別をつけるために
        @gifmgk = GifMgk.new      #パラメータを二種類用意する。
        ResourcePicture.stub(:png_to_gif).with(any_args).and_return(@gifmgk)
      end
      it '自身を保存している' do
        ResourcePicture.any_instance.should_receive(:save).with(any_args).exactly(1)
        res = @rp.store(Mgk.new)
      end
      it '画像ファイルの作成・更新機能で画像を保存している' do
        mgk = Mgk.new   #一回目の本画像保存は与えたオブジェクトを使って保存する。
        ResourcePicture.any_instance.should_receive(:store_picture).with(mgk).exactly(1)
        res = @rp.store(mgk)
      end
      it '自身にフォーマット変換対象かを問い合わせている' do
        ResourcePicture.any_instance.should_receive(:to_gif?).with(any_args).exactly(1)
        res = @rp.store(Mgk.new)
      end
      it '自身にGifフォーマット変換を依頼している' do
        ResourcePicture.should_receive(:png_to_gif).with(any_args).exactly(1)
        res = @rp.store(Mgk.new)
      end
      it '画像ファイルの作成・更新機能でgif画像を保存している' do
        #二回目の保存はgif変換の結果を渡す。
        ResourcePicture.any_instance.should_receive(:store_picture).with(@gifmgk).exactly(1)
        res = @rp.store(Mgk.new)
      end
    end
    context 'つつがなく終わるとき' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
#        ResourcePicture.any_instance.stub(:save).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:to_gif?).with(any_args).and_return(true)
        class GifMgk < Mgk ; end  #store_pictureは二回呼び出される。区別をつけるために
        @gifmgk = GifMgk.new      #パラメータを二種類用意する。
        ResourcePicture.stub(:png_to_gif).with(any_args).and_return(@gifmgk)
      end
      it 'Trueを返す' do
        res = @rp.store(Mgk.new)
        res.should be_true
      end
      it '自身が保存されている' do
        lambda {
          res = @rp.store(Mgk.new)
        }.should change ResourcePicture, :count
      end
    end
    context 'gif変換なしで、つつがなく終わるとき' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
        ResourcePicture.any_instance.stub(:save).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:to_gif?).with(any_args).and_return(false)
      end
      it 'Trueを返す' do
        res = @rp.store(Mgk.new)
        res.should be_true
      end
      it 'gif保存は呼ばれていない' do
        ResourcePicture.any_instance.should_receive(:store_picture).with(any_args).exactly(1)
        res = @rp.store(Mgk.new)
      end
    end
    #以下から例外ケース。処理先頭から失敗させていく
    context '自身の保存に失敗したとき' do
      before do
        ResourcePicture.any_instance.stub(:save).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        res = @rp.store(Mgk.new)
        res.should be_false
      end
      it '更新されていない' do
        @rp.store(Mgk.new)
        @rp.should be_a_new ResourcePicture
      end
    end
    context '画像の保存に失敗したとき' do
      before do
        ResourcePicture.any_instance.stub(:save).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:store_picture).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        res = @rp.store(Mgk.new)
        res.should be_false
      end
      it 'gif変換判定は呼ばれていない' do
        ResourcePicture.any_instance.should_not_receive(:to_gif?).with(any_args)
        res = @rp.store(Mgk.new)
      end
    end
    context 'gif変換に失敗したとき' do
      before do
        ResourcePicture.any_instance.stub(:save).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:to_gif?).with(any_args).and_return(true)
        ResourcePicture.stub(:png_to_gif).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        res = @rp.store(Mgk.new)
        res.should be_false
      end
      it 'gif画像の保存は呼ばれていない' do
        #本画像の保存があるので、一度は呼ばれる
        ResourcePicture.any_instance.should_receive(:store_picture).with(any_args).exactly(1)
        res = @rp.store(Mgk.new)
      end
    end
    context 'gif画像の保存に失敗したとき' do
      before do
        @mgk = Mgk.new
        ResourcePicture.any_instance.stub(:save).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:store_picture).with(@mgk).and_return(true)
        ResourcePicture.any_instance.stub(:to_gif?).with(any_args).and_return(true)
        class GifMgk < Mgk ; end  #store_pictureは二回呼び出される。区別をつけるために
        @gifmgk = GifMgk.new      #パラメータを二種類用意する。
        ResourcePicture.stub(:png_to_gif).with(any_args).and_return(@gifmgk)
        ResourcePicture.any_instance.stub(:store_picture).with(@gifmgk).and_return(false)
      end
      it 'Falseを返す' do
        res = @rp.store(@mgk)
        res.should be_false
      end
    end
  end
  
  describe '画像ファイルの作成・更新に於いて' do
    #PictureIo経由で画像を保存するための機能。ファイル名に自身のidを使うので事前に自身の保存が必要。
    before do
      PictureIO.resource_picture_io.class.stub(:subdirs).with(any_args).and_return(['', 'v', 'h', 'vh', 'thumbnail'])
      ResourcePicture.any_instance.stub(:h).with(any_args).and_return('data')
      ResourcePicture.any_instance.stub(:v).with(any_args).and_return('data')
      ResourcePicture.any_instance.stub(:vh).with(any_args).and_return('data')
      ResourcePicture.any_instance.stub(:thumbnail).with(any_args).and_return('data')
      @op = Factory :original_picture, :artist_id => @artist.id, :license_id => @license.id
      @rp = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id
    end
    context '事前チェック' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
        PictureIO.resource_picture_io.stub(:put).with(any_args).and_return(true)
      end
      it '画像ファイルの保存が5回呼ばれる' do
        PictureIO.resource_picture_io.should_receive(:put).with(any_args).exactly(5)
        res = @rp.store_picture(Mgk.new)
      end
      it '画像ファイルのベースへの保存が1回呼ばれる' do
        PictureIO.resource_picture_io.should_receive(:put).with('data', @rp.filename, '').exactly(1)
        res = @rp.store_picture(Mgk.new)
      end
      it '画像ファイルの垂直反転への保存が1回呼ばれる' do
        PictureIO.resource_picture_io.should_receive(:put).with('data', @rp.filename, 'v').exactly(1)
        res = @rp.store_picture(Mgk.new)
      end
      it '画像ファイルの水平反転への保存が1回呼ばれる' do
        PictureIO.resource_picture_io.should_receive(:put).with('data', @rp.filename, 'h').exactly(1)
        res = @rp.store_picture(Mgk.new)
      end
      it '画像ファイルの垂直水平反転への保存が1回呼ばれる' do
        PictureIO.resource_picture_io.should_receive(:put).with('data', @rp.filename, 'vh').exactly(1)
        res = @rp.store_picture(Mgk.new)
      end
      it '画像ファイルのサムネイルへの保存が1回呼ばれる' do
        PictureIO.resource_picture_io.should_receive(:put).with('data', @rp.filename, 'thumbnail').exactly(1)
        res = @rp.store_picture(Mgk.new)
      end
      it '垂直反転が1回呼ばれる' do
        ResourcePicture.any_instance.should_receive(:v).with(any_args).exactly(1)
        res = @rp.store_picture(Mgk.new)
      end
      it '水平反転が1回呼ばれる' do
        ResourcePicture.any_instance.should_receive(:h).with(any_args).exactly(1)
        res = @rp.store_picture(Mgk.new)
      end
      it '垂直水平反転が1回呼ばれる' do
        ResourcePicture.any_instance.should_receive(:vh).with(any_args).exactly(1)
        res = @rp.store_picture(Mgk.new)
      end
      it 'サムネイル化が1回呼ばれる' do
        ResourcePicture.any_instance.should_receive(:thumbnail).with(any_args).exactly(1)
        res = @rp.store_picture(Mgk.new)
      end
    end
    context 'つつがなく終わるとき' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
        PictureIO.resource_picture_io.stub(:put).with(any_args).and_return(true)
      end
      it 'Trueを返す' do
        res = @rp.store_picture(Mgk.new)
        res.should be_true
      end
    end
    context '例外ケース' do
      before do
        PictureIO.resource_picture_io.stub(:put).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        res = @rp.store_picture(Mgk.new)
        res.should be_false
      end
    end
    
  end
  
describe 'サムネイル変換に於いて' do
    #サムネイル化した画像データを返すが、スタブをおくので、リサイズと画像データ化を試みるかをチェックする
    before do
      ResourcePicture.stub(:resize).with(any_args).and_return(Mgk.new)
      @rp = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id
    end
    it 'サムネイルサイズに縮小した画像データを返す' do
      ResourcePicture.should_receive(:resize).with(any_args).exactly(1)
      @rp.thumbnail(Mgk.new)
    end
    it 'データが返る' do
      #全体スタブより
      @rp.thumbnail(Mgk.new).should eq 'data'
    end
  end
  
  describe '垂直反転変換に於いて' do
    #垂直反転した画像データを返すが、スタブをおくので、反転と画像データ化を試みるかをチェックする
    before do
      Mgk.any_instance.stub(:flip).with(any_args).and_return(Mgk.new)
      @rp = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id
    end
    it '垂直反転する' do
      Mgk.any_instance.should_receive(:flip).exactly(1)
      @rp.v(Mgk.new)
    end
    it '画像データ化する' do
      Mgk.any_instance.should_receive(:to_blob).exactly(1)
      @rp.v(Mgk.new)
    end
    it 'データが返る' do
      #全体スタブより
      @rp.v(Mgk.new).should eq 'data'
    end
  end
  
  describe '水平反転変換に於いて' do
    #水平反転した画像データを返すが、スタブをおくので、反転と画像データ化を試みるかをチェックする
    before do
      Mgk.any_instance.stub(:flop).with(any_args).and_return(Mgk.new)
      @rp = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id
    end
    it '水平反転する' do
      Mgk.any_instance.should_receive(:flop).exactly(1)
      @rp.h(Mgk.new)
    end
    it '画像データ化する' do
      Mgk.any_instance.should_receive(:to_blob).exactly(1)
      @rp.h(Mgk.new)
    end
    it 'データが返る' do
      #全体スタブより
      @rp.h(Mgk.new).should eq 'data'
    end
  end
  
  describe '垂直水平反転変換に於いて' do
    #垂直水平反転した画像データを返すが、スタブをおくので、反転と画像データ化を試みるかをチェックする
    before do
      Mgk.any_instance.stub(:flip).with(any_args).and_return(Mgk.new)
      Mgk.any_instance.stub(:flop).with(any_args).and_return(Mgk.new)
      @rp = Factory :resource_picture, :artist_id => @artist.id, :license_id => @license.id
    end
    it '垂直反転する' do
      Mgk.any_instance.should_receive(:flip).exactly(1)
      @rp.vh(Mgk.new)
    end
    it '水平反転する' do
      Mgk.any_instance.should_receive(:flop).exactly(1)
      @rp.vh(Mgk.new)
    end
    it '画像データ化する' do
      Mgk.any_instance.should_receive(:to_blob).exactly(1)
      @rp.vh(Mgk.new)
    end
    it 'データが返る' do
      #全体スタブより
      @rp.vh(Mgk.new).should eq 'data'
    end
  end
  
end

