# -*- encoding: utf-8 -*-
require 'spec_helper'

describe Speech do
  before do
    Factory :admin
    @license = Factory :license
    @user = Factory( :user_yas)
    @author = @user.author
    
    @panel = Factory :panel
    @speech_balloon_template = Factory :speech_balloon_template
    @speech_balloon = Factory :speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @speech_balloon_template.id
  end
  
  describe '検証に於いて' do
    it 'オーソドックスなデータなら通る' do
      @speech = Factory.build :speech, :speech_balloon_id => @speech_balloon.id
      @speech.should be_valid
    end
    
    context 'speech_balloon_idを検証するとき' do
      before do
        @speech = Factory.build :speech, :speech_balloon_id => @speech_balloon.id
      end
      it 'テストデータの確認' do
        @speech.speech_balloon_id = @speech_balloon.id
        @speech.should be_valid
      end
      it '数値でなければ失敗する' do
        @speech.speech_balloon_id = 'a'
        @speech.should_not be_valid
      end
    end
    context 'contentを検証するとき' do
    end
    context 'xを検証するとき' do
      before do
        @speech = Factory.build :speech, :speech_balloon_id => @speech_balloon.id
      end
      it 'テストデータの確認' do
        @speech.x = '1'
        @speech.should be_valid
      end
      it 'nullなら失敗する' do
        @speech.x = nil
        @speech.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @speech.x = 'a'
        @speech.should_not be_valid
      end
      it '0なら通る' do
        @speech.x = '0'
        @speech.should be_valid
      end
      it '負でも通る' do
        @speech.x = -1
        @speech.should be_valid
      end
    end
    context 'yを検証するとき' do
      before do
        @speech = Factory.build :speech, :speech_balloon_id => @speech_balloon.id
      end
      it 'テストデータの確認' do
        @speech.y = '1'
        @speech.should be_valid
      end
      it 'nullなら失敗する' do
        @speech.y = nil
        @speech.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @speech.y = 'a'
        @speech.should_not be_valid
      end
      it '0なら通る' do
        @speech.y = '0'
        @speech.should be_valid
      end
      it '負でも通る' do
        @speech.y = -1
        @speech.should be_valid
      end
    end
    context 'widthを検証するとき' do
      before do
        @speech = Factory.build :speech, :speech_balloon_id => @speech_balloon.id
      end
      it 'テストデータの確認' do
        @speech.width = 1
        @speech.should be_valid
      end
      it 'nullなら失敗する' do
        @speech.width = nil
        @speech.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @speech.width = 'a'
        @speech.should_not be_valid
      end
      it '0なら失敗する' do
        @speech.width = '0'
        @speech.should_not be_valid
      end
      it '負でも失敗する' do
        @speech.width = -1
        @speech.should_not be_valid
      end
    end
    context 'heightを検証するとき' do
      before do
        @speech = Factory.build :speech, :speech_balloon_id => @speech_balloon.id
      end
      it 'テストデータの確認' do
        @speech.height = '1'
        @speech.should be_valid
      end
      it 'nullなら失敗する' do
        @speech.height = nil
        @speech.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @speech.height = 'a'
        @speech.should_not be_valid
      end
      it '0なら失敗する' do
        @speech.height = '0'
        @speech.should_not be_valid
      end
      it '負でも失敗する' do
        @speech.height = -1
        @speech.should_not be_valid
      end
    end
    context 'settingsを検証するとき' do
      before do
        @speech = Factory.build :speech, :speech_balloon_id => @speech_balloon.id
      end
      it 'テストデータの確認' do
        @speech.settings = ''
        @speech.should be_valid
      end
    end
  end
  
end
